@@ -320,9 +320,9 @@ void prepare_for_binding(HTMLDocument *This, IMoniker *mon, DWORD flags)
     }
 
     if(This->window->mon) {
-        update_doc(This, UPDATE_TITLE|UPDATE_UI);
+        update_doc(This->doc_obj, UPDATE_TITLE|UPDATE_UI);
     }else {
-        update_doc(This, UPDATE_TITLE);
+        update_doc(This->doc_obj, UPDATE_TITLE);
         set_current_mon(This->window, mon, flags);
     }
 
@@ -424,7 +424,7 @@ HRESULT set_moniker(HTMLOuterWindow *window, IMoniker *mon, IUri *nav_uri, IBind
 
             task = heap_alloc(sizeof(docobj_task_t));
             task->doc = doc_obj;
-            hres = push_task(&task->header, set_progress_proc, NULL, doc_obj->basedoc.task_magic);
+            hres = push_task(&task->header, set_progress_proc, NULL, doc_obj->task_magic);
             if(FAILED(hres)) {
                 CoTaskMemFree(url);
                 return hres;
@@ -435,7 +435,7 @@ HRESULT set_moniker(HTMLOuterWindow *window, IMoniker *mon, IUri *nav_uri, IBind
         download_task->doc = doc_obj;
         download_task->set_download = set_download;
         download_task->url = url;
-        return push_task(&download_task->header, set_downloading_proc, set_downloading_task_destr, doc_obj->basedoc.task_magic);
+        return push_task(&download_task->header, set_downloading_proc, set_downloading_task_destr, doc_obj->task_magic);
     }
 
     return S_OK;
@@ -918,10 +918,9 @@ static HRESULT WINAPI PersistStreamInit_Load(IPersistStreamInit *iface, IStream
 
     prepare_for_binding(This, mon, FALSE);
     hres = set_moniker(This->window, mon, NULL, NULL, NULL, TRUE);
-    if(FAILED(hres))
-        return hres;
+    if(SUCCEEDED(hres))
+        hres = channelbsc_load_stream(This->window->pending_window, mon, pStm);
 
-    hres = channelbsc_load_stream(This->window->pending_window, mon, pStm);
     IMoniker_Release(mon);
     return hres;
 }
@@ -976,10 +975,9 @@ static HRESULT WINAPI PersistStreamInit_InitNew(IPersistStreamInit *iface)
 
     prepare_for_binding(This, mon, FALSE);
     hres = set_moniker(This->window, mon, NULL, NULL, NULL, FALSE);
-    if(FAILED(hres))
-        return hres;
+    if(SUCCEEDED(hres))
+        hres = channelbsc_load_stream(This->window->pending_window, mon, NULL);
 
-    hres = channelbsc_load_stream(This->window->pending_window, mon, NULL);
     IMoniker_Release(mon);
     return hres;
 }
