@@ -117,11 +117,11 @@ int    __cdecl _putw(int,FILE*);
 int    __cdecl _rmtmp(void);
 int    __cdecl _set_printf_count_output(int);
 int    __cdecl _setmaxstdio(int);
-int    __cdecl _snprintf(char*,size_t,const char*,...);
-int    __cdecl _snprintf_s(char*,size_t,size_t,const char*,...);
+int    WINAPIV _snprintf(char*,size_t,const char*,...);
+int    WINAPIV _snprintf_s(char*,size_t,size_t,const char*,...);
 char*  __cdecl _tempnam(const char*,const char*);
 int    __cdecl _unlink(const char*);
-int    __cdecl _scprintf(const char*,...);
+int    WINAPIV _scprintf(const char*,...);
 int    __cdecl _vscprintf(const char*,__ms_va_list);
 int    __cdecl _vsnprintf(char*,size_t,const char*,__ms_va_list);
 int    __cdecl _vsnprintf_s(char*,size_t,size_t,const char*,__ms_va_list);
@@ -153,15 +153,15 @@ int    __cdecl fgetpos(FILE*,fpos_t*);
 char*  __cdecl fgets(char*,int,FILE*);
 FILE*  __cdecl fopen(const char*,const char*);
 errno_t __cdecl fopen_s(FILE**,const char*,const char*);
-int    __cdecl fprintf(FILE*,const char*,...);
-int    __cdecl fprintf_s(FILE*,const char*,...);
+int    WINAPIV fprintf(FILE*,const char*,...);
+int    WINAPIV fprintf_s(FILE*,const char*,...);
 int    __cdecl fputc(int,FILE*);
 int    __cdecl fputs(const char*,FILE*);
 size_t __cdecl fread(void*,size_t,size_t,FILE*);
 size_t __cdecl fread_s(void*,size_t,size_t,size_t,FILE*);
 FILE*  __cdecl freopen(const char*,const char*,FILE*);
-int    __cdecl fscanf(FILE*,const char*,...);
-int    __cdecl fscanf_s(FILE*,const char*,...);
+int    WINAPIV fscanf(FILE*,const char*,...);
+int    WINAPIV fscanf_s(FILE*,const char*,...);
 int    __cdecl fseek(FILE*,__msvcrt_long,int);
 int    __cdecl _fseeki64(FILE*,__int64,int);
 int    __cdecl fsetpos(FILE*,fpos_t*);
@@ -172,23 +172,23 @@ int    __cdecl getc(FILE*);
 int    __cdecl getchar(void);
 char*  __cdecl gets(char*);
 void   __cdecl perror(const char*);
-int    __cdecl printf(const char*,...);
-int    __cdecl printf_s(const char*,...);
+int    WINAPIV printf(const char*,...);
+int    WINAPIV printf_s(const char*,...);
 int    __cdecl putc(int,FILE*);
 int    __cdecl putchar(int);
 int    __cdecl puts(const char*);
 int    __cdecl remove(const char*);
 int    __cdecl rename(const char*,const char*);
 void   __cdecl rewind(FILE*);
-int    __cdecl scanf(const char*,...);
-int    __cdecl scanf_s(const char*,...);
+int    WINAPIV scanf(const char*,...);
+int    WINAPIV scanf_s(const char*,...);
 void   __cdecl setbuf(FILE*,char*);
 int    __cdecl setvbuf(FILE*,char*,int,size_t);
-int    __cdecl sprintf(char*,const char*,...);
-int    __cdecl sprintf_s(char*,size_t,const char*,...);
-int    __cdecl _scprintf(const char *, ...);
-int    __cdecl sscanf(const char*,const char*,...);
-int    __cdecl sscanf_s(const char*,const char*,...);
+int    WINAPIV sprintf(char*,const char*,...);
+int    WINAPIV sprintf_s(char*,size_t,const char*,...);
+int    WINAPIV _scprintf(const char *, ...);
+int    WINAPIV sscanf(const char*,const char*,...);
+int    WINAPIV sscanf_s(const char*,const char*,...);
 FILE*  __cdecl tmpfile(void);
 char*  __cdecl tmpnam(char*);
 int    __cdecl ungetc(int,FILE*);
@@ -211,9 +211,9 @@ wint_t   __cdecl _getwc_nolock(FILE*);
 wchar_t* __cdecl _getws(wchar_t*);
 wint_t   __cdecl _putwc_nolock(wint_t,FILE*);
 int      __cdecl _putws(const wchar_t*);
-int      __cdecl _snwprintf(wchar_t*,size_t,const wchar_t*,...);
-int      __cdecl _snwprintf_s(wchar_t*,size_t,size_t,const wchar_t*,...);
-int      __cdecl _scwprintf(const wchar_t*,...);
+int      WINAPIV _snwprintf(wchar_t*,size_t,const wchar_t*,...);
+int      WINAPIV _snwprintf_s(wchar_t*,size_t,size_t,const wchar_t*,...);
+int      WINAPIV _scwprintf(const wchar_t*,...);
 wint_t   __cdecl _ungetwc_nolock(wint_t,FILE*);
 int      __cdecl _vscwprintf(const wchar_t*,__ms_va_list);
 int      __cdecl _vscwprintf_p_l(const wchar_t*,_locale_t,__ms_va_list);
@@ -235,21 +235,21 @@ wint_t   __cdecl fgetwc(FILE*);
 wchar_t* __cdecl fgetws(wchar_t*,int,FILE*);
 wint_t   __cdecl fputwc(wint_t,FILE*);
 int      __cdecl fputws(const wchar_t*,FILE*);
-int      __cdecl fwprintf(FILE*,const wchar_t*,...);
-int      __cdecl fwprintf_s(FILE*,const wchar_t*,...);
+int      WINAPIV fwprintf(FILE*,const wchar_t*,...);
+int      WINAPIV fwprintf_s(FILE*,const wchar_t*,...);
 int      __cdecl fputws(const wchar_t*,FILE*);
-int      __cdecl fwscanf(FILE*,const wchar_t*,...);
-int      __cdecl fwscanf_s(FILE*,const wchar_t*,...);
+int      WINAPIV fwscanf(FILE*,const wchar_t*,...);
+int      WINAPIV fwscanf_s(FILE*,const wchar_t*,...);
 wint_t   __cdecl getwc(FILE*);
 wint_t   __cdecl getwchar(void);
 wchar_t* __cdecl getws(wchar_t*);
 wint_t   __cdecl putwc(wint_t,FILE*);
 wint_t   __cdecl putwchar(wint_t);
 int      __cdecl putws(const wchar_t*);
-int      __cdecl swprintf(wchar_t*,const wchar_t*,...);
-int      __cdecl swprintf_s(wchar_t*,size_t,const wchar_t*,...);
-int      __cdecl swscanf(const wchar_t*,const wchar_t*,...);
-int      __cdecl swscanf_s(const wchar_t*,const wchar_t*,...);
+int      WINAPIV swprintf(wchar_t*,const wchar_t*,...);
+int      WINAPIV swprintf_s(wchar_t*,size_t,const wchar_t*,...);
+int      WINAPIV swscanf(const wchar_t*,const wchar_t*,...);
+int      WINAPIV swscanf_s(const wchar_t*,const wchar_t*,...);
 wint_t   __cdecl ungetwc(wint_t,FILE*);
 int      __cdecl vfwprintf(FILE*,const wchar_t*,__ms_va_list);
 int      __cdecl vfwprintf_s(FILE*,const wchar_t*,__ms_va_list);
@@ -257,10 +257,10 @@ int      __cdecl vswprintf(wchar_t*,const wchar_t*,__ms_va_list);
 int      __cdecl vswprintf_s(wchar_t*,size_t,const wchar_t*,__ms_va_list);
 int      __cdecl vwprintf(const wchar_t*,__ms_va_list);
 int      __cdecl vwprintf_s(const wchar_t*,__ms_va_list);
-int      __cdecl wprintf(const wchar_t*,...);
-int      __cdecl wprintf_s(const wchar_t*,...);
-int      __cdecl wscanf(const wchar_t*,...);
-int      __cdecl wscanf_s(const wchar_t*,...);
+int      WINAPIV wprintf(const wchar_t*,...);
+int      WINAPIV wprintf_s(const wchar_t*,...);
+int      WINAPIV wscanf(const wchar_t*,...);
+int      WINAPIV wscanf_s(const wchar_t*,...);
 #endif /* _WSTDIO_DEFINED */
 
 #endif /* _STDIO_DEFINED */
