@@ -68,6 +68,7 @@ extern const char *wine_dbgstr_wn( const WCHAR *str, int n );
 extern const char *wine_dbgstr_guid( const GUID *guid );
 extern const char *wine_dbgstr_rect( const RECT *rect );
 static inline const char *wine_dbgstr_w( const WCHAR *s ) { return wine_dbgstr_wn( s, -1 ); }
+extern const char *wine_dbgstr_longlong( ULONGLONG ll );
 
 /* strcmpW is available for tests compiled under Wine, but not in standalone
  * builds under Windows, so we reimplement it under a different name. */
@@ -86,7 +87,7 @@ static inline int winetest_strcmpW( const WCHAR *str1, const WCHAR *str2 )
 #define START_TEST(name) void func_##name(void)
 #endif
 
-#if defined(__x86_64__) && defined(__GNUC__) && defined(__WINE_USE_MSVCRT)
+#if (defined(__x86_64__) || defined(__aarch64__)) && defined(__GNUC__) && defined(__WINE_USE_MSVCRT)
 #define __winetest_cdecl __cdecl
 #define __winetest_va_list __builtin_ms_va_list
 #else
@@ -177,7 +178,7 @@ extern void __winetest_cdecl winetest_trace( const char *msg, ... ) WINETEST_PRI
 #include <stdio.h>
 #include <excpt.h>
 
-#if defined(__x86_64__) && defined(__GNUC__) && defined(__WINE_USE_MSVCRT)
+#if (defined(__x86_64__) || defined(__aarch64__)) && defined(__GNUC__) && defined(__WINE_USE_MSVCRT)
 # define __winetest_va_start(list,arg) __builtin_ms_va_start(list,arg)
 # define __winetest_va_end(list) __builtin_ms_va_end(list)
 #else
@@ -541,6 +542,19 @@ const char *wine_dbgstr_rect( const RECT *rect )
     return res;
 }
 
+const char *wine_dbgstr_longlong( ULONGLONG ll )
+{
+    char *res;
+
+    res = get_temp_buffer( 17 );
+    if (sizeof(ll) > sizeof(unsigned long) && ll >> 32)
+        sprintf( res, "%lx%08lx", (unsigned long)(ll >> 32), (unsigned long)ll );
+    else
+        sprintf( res, "%lx", (unsigned long)ll );
+    release_temp_buffer( res, strlen(res) + 1 );
+    return res;
+}
+
 /* Find a test by name */
 static const struct test *find_test( const char *name )
 {
